# Create your views here.
from django.http import HttpResponse
from django.template.loader import get_template
from django.template import Context
from django.template import RequestContext
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from app_2015.models import Kuva
from app_2015.models import Kategoria
from app_2015.models import KuvaPari
from app_2015.models import LadattuKuva
from app_2015.forms import UploadImageForm
from app_2015.forms import CreateCategoryForm
import random
import operator

def main_template(request, non):
	return render_to_response('main.html', {'kategoriat': Kategoria.objects.all() })
	
def admin(request, non):
	return render_to_response('admin.html', {'kategoriat': Kategoria.objects.all() }, context_instance=RequestContext(request))
	
def get_something_rated(request, non):
	return render_to_response('get_something_rated.html', context_instance=RequestContext(request))
	
def introduction(request, non):
	return render_to_response('introduction.html', context_instance=RequestContext(request))
	
def get_images(request, kategoria_id=1):	
	images = Kuva.objects.filter(kategoria = kategoria_id)
	
	if len(images) < 2:
		return HttpResponse("")
		
	firstIndex = random.randint(0, len(images) - 1)
	stringToReturn = images[firstIndex].sijainti + "  " + str(images[firstIndex].id) + "  " + images[firstIndex].nimi
	secondIndex = random.randint(0, len(images) - 1)
	while firstIndex == secondIndex:
		secondIndex = random.randint(0, len(images) - 1)
	stringToReturn += "  " + images[secondIndex].sijainti + "  " + str(images[secondIndex].id) + "  " + images[secondIndex].nimi
	
	stringToReturn += "  " + get_category_information(kategoria_id)
	return HttpResponse(stringToReturn)
	
def get_category_information(category_id):
	category = Kategoria.objects.get(id = category_id)
	return category.nimi + "  " + category.otsikko + "  " + category.kuvaus
	
def send_rating(request, message):
	winnerID = int(message.split("_")[0])
	loserID = int(message.split("_")[1])
	
	kuvapari = KuvaPari.objects.filter(kuva1__in = [winnerID, loserID]).filter(kuva2__in = [winnerID, loserID])[0]
	if kuvapari.kuva1.id == winnerID:
		kuvapari.kuva1_aanet += 1
	elif kuvapari.kuva2.id == winnerID:
		kuvapari.kuva2_aanet += 1
	else:
		print("error")
		
	kuvapari.save()
	return HttpResponse("");
	
def add_image(kuvan_nimi, kategoria_id, kuvan_sijainti):
	old_images = Kuva.objects.filter(kategoria = kategoria_id)
	new_image = Kuva(nimi = kuvan_nimi, kategoria = Kategoria.objects.filter(id = kategoria_id)[0], sijainti = kuvan_sijainti)
	new_image.save()
	
	for old_image in old_images:
		if old_image == new_image:
			continue
		kuvapari = KuvaPari(kuva1 = old_image, kuva2 = new_image, kategoria = Kategoria.objects.filter(id = kategoria_id)[0])
		kuvapari.save()

def upload_image(request):
	if request.method == "POST":
		form = UploadImageForm(request.POST, request.FILES)
		if form.is_valid():
			image_to_upload = LadattuKuva(tiedosto = request.FILES['tiedosto'])
			image_to_upload.save()
			
			if not request.POST.get('txt_name'):
				return HttpResponse("error", mimetype="text/plain")
			
			if not request.POST.get('ddl_category'):
				return HttpResponse("error", mimetype="text/plain")
				
			add_image(str(request.POST['txt_name']), int(request.POST['ddl_category']), "assets/img/" + request.FILES['tiedosto'].name)
			
			return render_to_response('admin.html', {'kategoriat': Kategoria.objects.all() }, context_instance=RequestContext(request))
	return HttpResponse("error", mimetype="text/plain")
		
def create_category(request):
	if request.method == "POST":
		form = CreateCategoryForm(request.POST, request.FILES)
		if form.is_valid():
			image_to_upload = LadattuKuva(tiedosto = request.FILES['cover_image'])
			image_to_upload.save()
			
			new_category = Kategoria(nimi = str(request.POST['txt_category_name']), otsikko = str(request.POST['txt_category_title']), kuvaus = str(request.POST['txt_category_description']), kansikuva_sijainti = "assets/img/" + request.FILES['cover_image'].name)
			new_category.save()
			
			return render_to_response('admin.html', {'kategoriat': Kategoria.objects.all() }, context_instance=RequestContext(request))
	return HttpResponse("error", mimetype="text/plain")
	
def get_votes_for_image(image_id):
	kuvaparit = KuvaPari.objects.filter(kuva1 = image_id)
	kuvaparit2 = KuvaPari.objects.filter(kuva2 = image_id)
	
	winning_votes = 0
	losing_votes = 0
	
	for kuvapari in kuvaparit:
		winning_votes += kuvapari.kuva1_aanet
		losing_votes += kuvapari.kuva2_aanet
	for kuvapari in kuvaparit2:
		winning_votes += kuvapari.kuva2_aanet
		losing_votes += kuvapari.kuva1_aanet
		
	return [winning_votes, losing_votes]
	
def create_statistics_string_for_image(image_id):
	image = Kuva.objects.get(id = image_id)
	votes = get_votes_for_image(image_id)
	
	return str(image_id) + "  " + str(votes[0]) + "  " + str(votes[1]) + "  " + image.sijainti + "  " + image.nimi + "  "
	
def create_statistics_string(request, category_id):
	images = sort_images(Kuva.objects.filter(kategoria = category_id))
	
	statistics_string = ""
	for image in images:
		statistics_string += create_statistics_string_for_image(image.id)
		
	return HttpResponse(statistics_string.strip())
	
def sort_images(images):
	images_and_percentages = {}
		
	for image in images:
		votes = get_votes_for_image(image.id)
		percentage = 0
		if votes[0] + votes[1] > 0:
			percentage = float(votes[0]) / (votes[0] + votes[1])
				
		images_and_percentages[image] = percentage
		
	return sorted(images_and_percentages, key = images_and_percentages.get, reverse = True)
			
	
def get_votes_for_images(request, image1_id, image2_id):
	kuvapari = KuvaPari.objects.filter(kuva1__in = [image1_id, image2_id]).filter(kuva2__in = [image1_id, image2_id])[0]

	if int(kuvapari.kuva1.id) == int(image1_id) and int(kuvapari.kuva2.id) == int(image2_id):
		return HttpResponse(str(kuvapari.kuva1_aanet) + " " + str(kuvapari.kuva2_aanet))
	elif int(kuvapari.kuva2.id) == int(image1_id) and int(kuvapari.kuva1.id) == int(image2_id):
		return HttpResponse(str(kuvapari.kuva2_aanet) + " " + str(kuvapari.kuva1_aanet))
	else:
		print("error at 'get_votes_for_images'")
	return HttpResponse("0 0")